package org.syncloud.facade;

import org.syncloud.facade.dao.AccountsDao;
import org.syncloud.facade.dao.ConfigReader;
import org.syncloud.facade.dao.ConfigWriter;
import org.syncloud.facade.dao.SyncDao;
import org.syncloud.facade.dao.io.ClasspathReader;
import org.syncloud.facade.dao.io.FileReader;
import org.syncloud.facade.dao.io.FileWriter;
import org.syncloud.facade.dao.io.Reader;
import org.syncloud.facade.dao.json.AccountParser;
import org.syncloud.facade.dao.json.StorageMetadataReader;
import org.syncloud.facade.dao.json.SyncParser;
import org.syncloud.facade.dao.json.UserConfigJsonParser;
import org.syncloud.facade.proxy.DirectFoldersProxy;
import org.syncloud.facade.proxy.FoldersProxy;
import syncloud.core.Paths;
import syncloud.storage.Account;
import syncloud.storage.IStorageMetadata;
import syncloud.storage.Sync;
import syncloud.storage.UserConfig;

import java.io.File;
import java.util.List;

public class CoreFactory {

    private static CoreFactory instance;

    public static final String USER_HOME = System.getProperty("user.home");
//    public static final String USER_SYNCLOUD_FOLDER = Paths.USER_SYNCLOUD_FOLDER;
    public static final String USER_CONFIG_FOLDER = "config";

    public static final String USER_CONFIG_FILE_NAME = "config.json";
    public static final String STORAGES_CONFIG_FILE_NAME = "storages.json";
    public static final String SYNC_CONFIG_FILE_NAME = "syncs.json";
    public static final String ACCOUNTS_CONFIG_FILE_NAME = "accounts.json";

    private ConfigurationManager configuration;
    private StorageManager storageManager;
    private FoldersProxy foldersProxy;
    private AccountsDao accountsDao;

    private AuthenticationManager authenticationManager;
    private ConfigReader<List<IStorageMetadata>> metadataConfigReader;
    private SyncDao syncDao;
    private File userSyncloudPath;


    public static synchronized CoreFactory getInstance() {
        if (instance == null) {
            instance = new CoreFactory();
            instance.init();
        }
        return instance;
    }

    public void init() {

        userSyncloudPath = getUserSyncloudPath(USER_HOME, Paths.USER_SYNCLOUD_FOLDER);

        String configFolder = getUserConfigFolderPath(userSyncloudPath, USER_CONFIG_FOLDER);

        configuration = new ConfigurationManager(
                new ConfigReader<UserConfig>(
                        new FileReader(configFolder, USER_CONFIG_FILE_NAME),
                        new UserConfigJsonParser()),
                new ConfigWriter<UserConfig>(
                        new FileWriter(configFolder, USER_CONFIG_FILE_NAME),
                        new UserConfigJsonParser()));

        metadataConfigReader = new ConfigReader<List<IStorageMetadata>>(
                getStorageConfigFileReader(USER_CONFIG_FOLDER, STORAGES_CONFIG_FILE_NAME),
                new StorageMetadataReader());

        storageManager = createStorageManager();
        foldersProxy = new DirectFoldersProxy(storageManager);

        AccountParser accountParser = new AccountParser();
        ConfigReader<List<Account>> configReader = new ConfigReader<List<Account>>(
                new FileReader(configFolder, ACCOUNTS_CONFIG_FILE_NAME),
                accountParser);

        ConfigWriter<List<Account>> configWriter = new ConfigWriter<List<Account>>(
                new FileWriter(configFolder, ACCOUNTS_CONFIG_FILE_NAME),
                accountParser);
        accountsDao = new AccountsDao(configReader, configWriter);

        authenticationManager = new AuthenticationManager(configuration, storageManager, accountsDao);


        SyncParser syncParser = new SyncParser();
        syncDao = new SyncDao(
                new ConfigReader<List<Sync>>(
                        new FileReader(configFolder, SYNC_CONFIG_FILE_NAME),
                        syncParser),
                new ConfigWriter<List<Sync>>(
                        new FileWriter(configFolder, SYNC_CONFIG_FILE_NAME),
                        syncParser));
    }

    protected String getUserConfigFolderPath(File userSyncloudFolder, String userConfigFolder) {
        return new File(userSyncloudFolder, userConfigFolder).getAbsolutePath();
    }

    protected File getUserSyncloudPath(String home, String userSyncloudFolder) {
        return new File(home, userSyncloudFolder);
    }

    protected Reader getStorageConfigFileReader(String configFolder, String storagesConfigFileName) {
        return new ClasspathReader(storagesConfigFileName);
    }

    protected StorageManager createStorageManager() {
        return new StorageManager(metadataConfigReader.read());
    }

    public ConfigurationManager getConfiguration() {
        return configuration;
    }

    public StorageManager getStorageManager() {
        return storageManager;
    }

    public FoldersProxy getFoldersProxy() {
        return foldersProxy;
    }

    public AccountsDao getAccountsDao() {
        return accountsDao;
    }

    public AuthenticationManager getAuthenticationManager() {
        return authenticationManager;
    }

    public SyncDao getSyncDao() {
        return syncDao;
    }

    public File getUserSyncloudPath() {
        return userSyncloudPath;
    }
}
