package org.syncloud.facade;

import syncloud.core.CoreException;
import syncloud.storage.IStorage;
import syncloud.storage.IStorageMetadata;
import syncloud.storage.StorageKey;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class StorageManager {

    private Map<StorageKey, IStorage> storages = new HashMap<StorageKey, IStorage>();
    private HashMap<String, IStorageMetadata> nameToMeta;

    public StorageManager(List<IStorageMetadata> metadata) {
        if (metadata == null)
            throw new CoreException("storage configuration is not found");
        init(metadata);
    }

    private void init(List<IStorageMetadata> metadata) {
        nameToMeta = new HashMap<String, IStorageMetadata>();
        for (IStorageMetadata meta : metadata) {
            nameToMeta.put(meta.getName(), meta);
        }
    }

    public IStorage getStorage(StorageKey key) throws CoreException {

        if (storages.containsKey(key))
            return storages.get(key);

        if (!nameToMeta.containsKey(key.getStorageId()))

            throw new CoreException("unknown storage with id: " + key.getStorageId());

        try {
            Class<? extends IStorage> storageClass = nameToMeta.get(key.getStorageId()).getStorageClass();
            IStorage storage = storageClass.getConstructor(StorageKey.class).newInstance(key);
            storages.put(key, storage);
            return storage;
        } catch (InstantiationException e) {
            throw new CoreException("unable to create storage with the following info: " + key);
        } catch (IllegalAccessException e) {
            throw new CoreException("unable to access storage with the following info: " + key);
        } catch (NoSuchMethodException e) {
            throw new CoreException("unable to construct storage with the following info: " + key + ", missing constructor signature: " + e.getMessage());
        } catch (InvocationTargetException e) {
            throw new CoreException("unable to create storage with the following info: " + key);
        }

    }

    public List<IStorageMetadata> getMetadata() {
        return new ArrayList<IStorageMetadata>(nameToMeta.values());
    }
}